from enum import IntEnum


# noinspection SpellCheckingInspection
class ServerID(IntEnum):
    BAHAMUT: int = 1
    SHIVA: int = 2
    TITAN: int = 3
    RAMUH: int = 4
    PHOENIX: int = 5
    CARBUNCLE: int = 6
    FENRIR: int = 7
    SYLPH: int = 8
    VALEFOR: int = 9
    ALEXANDER: int = 10
    LEVIATHAN: int = 11
    ODIN: int = 12
    IFRIT: int = 13
    DIABOLOS: int = 14
    CAITSITH: int = 15
    QUETZALCOATL: int = 16
    SIREN: int = 17
    UNICORN: int = 18
    GILGAMESH: int = 19
    RAGNAROK: int = 20
    PANDEMONIUM: int = 21
    GARUDA: int = 22
    CERBERUS: int = 23
    KUJATA: int = 24
    BISMARCK: int = 25
    SERAPH: int = 26
    LAKSHMI: int = 27
    ASURA: int = 28
    MIDGARDSORMR: int = 29
    FAIRY: int = 30
    REMORA: int = 31
    HADES: int = 32
